#ifndef _GMIDIIN_H
#define _GMIDIIN_H


#include "GMidiCommon.h"

#define CC_BANK_MSB    0
#define CC_MODULATION  1
#define CC_VOLUME      7
#define CC_PAN        10
#define CC_EXPRESSION 13
#define CC_BANK_LSB   32
#define CC_SUSTAIN    64
#define CC_PORTAMENTO 65
#define CC_REVERB     91
#define CC_CHORUS     93


#define SYSEX_NO_OVERFLOW 0x00
#define SYSEX_OVERFLOW    0x01

#define MESSAGE_BUFFER_SIZE 8

// Set buffer used to store incoming bytes and initialize.
// It is the users responsibility to pre-define the buffer as volatile
// if necessary (i.e., used with AVR interrupts)


class GMidiin
{
	public:
		GMidiin();
		void begin();
		void stop();
		void sendByte (const unsigned char);
		void setNoteOnFunc(void (*funcPtr)(int channel, int noteNo, int velocity));
		void setNoteOffFunc(void (*funcPtr)(int channel, int noteNo, int velocity));
		void setCcFunc(void (*funcPtr)(int channel, int ccNo, int value));
		void setPrgChangeFunc(void (*funcPtr)(int channel, int prgNo));
		void setPitchBendFunc(void (*funcPtr)(int channel, int value));
		void setSysexFunc(void (*funcPtr)(volatile unsigned char *sysex, int len, int overflow));

	private:
		void init ();
		void (*noteOnCallback)(int channel, int noteNo, int velocity);
		void (*noteOffCallback)(int channel, int noteNo, int velocity);
		void (*ccCallback)(int channel, int ccNo, int velocity);
		void (*prgChangeCallback)(int channel, int prgNo);
		void (*pitchBendCallback)(int channel, int value);
		void (*sysexCallback)(volatile unsigned char *sysex, int len, int overflow);
		volatile unsigned char *messageBuffer;
		volatile unsigned char messagePointer;
		volatile unsigned char messageOverflow;
};

extern GMidiin Midiin;

#endif // _GMIDIIN_H
