#include "GMidiout.h"


ISR(USART_TX_vect)
{
	Midiout.sendByte();
}


GMidiout::GMidiout()
{
}


void GMidiout::noteOn(unsigned char channel, unsigned char noteNo, unsigned char velocity)
{
	GMidiout::setSendByte(STATUS_NOTEON | channel, ON_FULL_WAIT);
	GMidiout::setSendByte(noteNo, ON_FULL_WAIT);
	GMidiout::setSendByte(velocity, ON_FULL_WAIT);
}


void GMidiout::noteOff(unsigned char channel, unsigned char noteNo, unsigned char velocity)
{
	GMidiout::setSendByte(STATUS_NOTEOFF | channel, ON_FULL_WAIT);
	GMidiout::setSendByte(noteNo, ON_FULL_WAIT);
	GMidiout::setSendByte(velocity, ON_FULL_WAIT);
}


void GMidiout::cc(unsigned char channel, unsigned char ccNo, unsigned char value)
{
	GMidiout::setSendByte(STATUS_CC | channel, ON_FULL_WAIT);
	GMidiout::setSendByte(ccNo, ON_FULL_WAIT);
	GMidiout::setSendByte(value, ON_FULL_WAIT);
}


void GMidiout::prgChange(unsigned char channel, unsigned char prgNo)
{
	GMidiout::setSendByte(STATUS_PRGCHANGE | channel, ON_FULL_WAIT);
	GMidiout::setSendByte(prgNo, ON_FULL_WAIT);
}


void GMidiout::pitchBend(unsigned char channel, int value)
{
	GMidiout::setSendByte(STATUS_PITCHBEND | channel, ON_FULL_WAIT);
	GMidiout::setSendByte((unsigned char)(value % 0x7F), ON_FULL_WAIT);
	GMidiout::setSendByte((unsigned char)(value / 0x7F), ON_FULL_WAIT);
}


void GMidiout::sysex(unsigned char *sysex, unsigned char len)
{
	int i;
	for (i=0; i<len; i++)
		GMidiout::setSendByte(outBuffer[i], ON_FULL_WAIT);
}


void GMidiout::sendByte()
{
	if (writePtr != readPtr) {
		UDR0 = outBuffer[readPtr++];
		if (readPtr == MIDIOUT_BUFFER_SIZE)
			readPtr = 0;
	}
}


unsigned char GMidiout::setSendByte(unsigned char byte, unsigned char onFull)
{
	while (((writePtr + 1) % MIDIOUT_BUFFER_SIZE) == readPtr) {
		if (onFull == ON_FULL_RETURN)
			return MIDIOUT_SEND_FULL;
	}

	outBuffer[writePtr++] = byte;

	if (writePtr == MIDIOUT_BUFFER_SIZE)
		writePtr = 0;

	if (UCSR0A & (1<<UDRE0))
		GMidiout::sendByte();

	return MIDIOUT_SEND_OK;
}


void GMidiout::begin()
{
	outBuffer = (volatile unsigned char *)malloc(MIDIOUT_BUFFER_SIZE);
	writePtr = 0;
	readPtr = 0;

	// Set baud rate to 31,250. Requires modification if clock speed is not 16MHz.
	UBRR0H = ((F_CPU / 16 + MIDI_BAUD_RATE / 2) / MIDI_BAUD_RATE - 1) >> 8;
	UBRR0L = ((F_CPU / 16 + MIDI_BAUD_RATE / 2) / MIDI_BAUD_RATE - 1);

	// enable tx
	UCSR0B |= _BV(TXEN0);
	
	// USART TX interrupt enable bit on
	UCSR0B |= _BV(TXCIE0);

}


GMidiout Midiout = GMidiout();
