// WiiMidi 
// ドラムロール
#include "WiiRemote.h"
#include "WMDDrumRoll.h"
#include "setMidiOUt.h"

extern WiiRemote wiiremote;


// MIDI チャンネル
#define MIDICHANNEL		9		// ch 10

// スネアの音量設定に使用するNRPNの値
#define NRPN_P5LV	0x21		// ES1 part 5(Level)

// クラッシュシンバルのnote
#define	SMP_CLASHCYMBAL	12*2	// ES1 part 1


// コンストラクタ
WMDDrumRoll::WMDDrumRoll()
{
	mRollFlug = 0;
}

void WMDDrumRoll::task()
{
	if(mRollFlug == 0)
	{
		// Bボタンが押されている間処理を行う
		if(wiiremote.buttonPressed(WIIREMOTE_B) == true)
		{
			mRollFlug = 1;
		}
	}
	else
	{
		if(wiiremote.buttonPressed(WIIREMOTE_B) == false)
		{
			// Bボタンを離したタイミングで、ロール音量を０にし
			// クラッシュシンバルを鳴らす。
			setNRPN(NRPN_P5LV,0);
			Midiout.noteOn( MIDICHANNEL,SMP_CLASHCYMBAL,127);
			Midiout.noteOff(MIDICHANNEL,SMP_CLASHCYMBAL,127);
			mRollFlug = 0;
		}
		else
		{
			// Bボタンを押している間、リモコンの傾きに応じて
			// ドラムロールの音量を変える
			double y = (double) wiiremote.Report.Accel.Y;
			unsigned char yy = (unsigned char)(map(y*100,-100,+100,127,0));
			setNRPN(NRPN_P5LV,yy);

		}
	}
}

// NRPN送信
void WMDDrumRoll::setNRPN(unsigned char kind,unsigned char value)
{
	Midiout.cc(9,0x63,0x05);
	Midiout.cc(9,0x62,kind);
	Midiout.cc(9,0x06,value);
	Midiout.cc(9,0x26,0);
}
