// WiiMidi モード
// エフェクタ設定
#include "WiiRemote.h"
#include "WMDEffect.h"
#include "setMidiOUt.h"


extern WiiRemote wiiremote;

// MIDI チャンネル
#define MIDICHANNEL		9		// ch 10

// ES-1のエフェクト種類
#define FX_WAH		10
#define FX_RESO		9
#define FX_ISOR		8
#define FX_DECI		7
#define FX_DIST		6
#define FX_COMP		5
#define FX_PITCH	4
#define FX_RING		3
#define FX_PHAS		2
#define FX_FLG		1
#define FX_REV		0

// ES-1の設定用NRPN
#define FX_NRPN_AUX_FX	0x54
#define FX_NRPN_SETFX	0x62
#define FX_NRPN_FX1		0x63
#define FX_NRPN_FX2		0x64


// クラッシュシンバルのnote
#define	SMP_CLASHCYMBAL	12*2	// ES1 part 1

// コンストラクタ
WMDEffect::WMDEffect()
{
	mEffectFlug = 0;
	mEffectType = 0;
}

void WMDEffect::task()
{

	// Bボタンを押している間、Audio inにエフェクトをかける
	if(mEffectFlug == 0)
	{
		// Bボタンが押されたらエフェクト設定
		if(wiiremote.buttonPressed(WIIREMOTE_B) == true)
		{
			setNRPN(FX_NRPN_AUX_FX,1);
			mEffectFlug = 1;
		}
	}
	else
	{
		// Bボタンを離したらエフェクト設定を外す
		if(wiiremote.buttonPressed(WIIREMOTE_B) == false)
		{
			setNRPN(FX_NRPN_AUX_FX,0);
			mEffectFlug = 0;
		}
	}

	// Ａボタン押したらクラッシュシンバル
	if(wiiremote.buttonPressed(WIIREMOTE_A) == true)
	{
		Midiout.noteOn( MIDICHANNEL,SMP_CLASHCYMBAL,127);
		Midiout.noteOff(MIDICHANNEL,SMP_CLASHCYMBAL,127);
	}
	
	// 十字キー、１，２キーでエフェクタ設定
	if(wiiremote.buttonClicked(WIIREMOTE_DOWN) == true)
	{	SetEffect(FX_WAH);	}

	if(wiiremote.buttonClicked(WIIREMOTE_LEFT) == true)
	{	SetEffect(FX_RESO);	}

	if(wiiremote.buttonClicked(WIIREMOTE_UP) == true)
	{	SetEffect(FX_REV);	}

	if(wiiremote.buttonClicked(WIIREMOTE_RIGHT) == true)
	{	SetEffect(FX_ISOR);	}

	if(wiiremote.buttonClicked(WIIREMOTE_ONE) == true)
	{	SetEffect(FX_DECI);	}

	if(wiiremote.buttonClicked(WIIREMOTE_TWO) == true)
	{	SetEffect(FX_DIST);	}

	// Ｂボタンを押している間、エフェクタのパラメータを変更する
	if(mEffectFlug != 0)
	{
		double x = (double) wiiremote.Report.Accel.X;
		double y = (double) wiiremote.Report.Accel.Y;
		int xx = (int)(map(x*100,-100,+100,127,0));
		int yy = (int)(map(y*100,-100,+100,127,0));
		SetFxParam(xx,yy);
	}

}

// AUXのエフェクタ ON/OFF設定
void WMDEffect::SetEffect(unsigned char type)
{
	mEffectType = type;
	setNRPN(FX_NRPN_SETFX,mEffectType);
}

// エフェクタのパラメータ変更
void WMDEffect::SetFxParam(int x,int y)
{

// エフェクタにより、傾きとパラメータの関係を変える
	switch(mEffectType)
	{
	case FX_WAH:
		setNRPN(FX_NRPN_FX1,0);
		setNRPN(FX_NRPN_FX2,y);
		break;
	case FX_RESO:
		setNRPN(FX_NRPN_FX1,y);
		setNRPN(FX_NRPN_FX2,127 - y);
		break;
	case FX_REV:
		setNRPN(FX_NRPN_FX1,y);
		setNRPN(FX_NRPN_FX2,y);
		break;
	case FX_ISOR:
		setNRPN(FX_NRPN_FX1,40);
		setNRPN(FX_NRPN_FX2,y);
		break;
	case FX_DECI:
		y=127-y;
		if(y<64)
		{
			setNRPN(FX_NRPN_FX1,64);
			setNRPN(FX_NRPN_FX2,64 + y);
		}
		else
		{
			setNRPN(FX_NRPN_FX1,y);
			setNRPN(FX_NRPN_FX2,127);
		}
		break;
	case FX_DIST:
		setNRPN(FX_NRPN_FX1,y);
		setNRPN(FX_NRPN_FX2,127 - x);
		break;
	}
}


// NRPN送信
void WMDEffect::setNRPN(unsigned char kind,unsigned char value)
{
	Midiout.cc(9,0x63,0x05);
	Midiout.cc(9,0x62,kind);
	Midiout.cc(9,0x06,value);
	Midiout.cc(9,0x26,0);
}
