#include "WiiRemote.h"
#include "VectorAnalyze.h"

#include "wiiMidimode.h"
#include "WMDSwingSample.h"
#include "WMDDrumRoll.h"
#include "WMDEffect.h"

#include "setMidiOUt.h"

#define SERIAL_DEBUG 0


// 振り回し、振り下ろし判定ルーチン
VectorAnalyze	mVector;

// Wiiリモコン使用ライブラリ
WiiRemote wiiremote;

// WiiMidi・個別コーディング
WiiMidi *pmWiiMidi;

// サンプル再生・ドラムロール・エフェクタ
WMDSwingSample	mWMDSwingSample;
WMDDrumRoll		mWMDDrumRoll;
WMDEffect		mWMDEffect;


// モード
char mMode;


void setup()
{
#if SERIAL_DEBUG
  Serial.begin(57600);
#endif

// オブジェクト初期化
	wiiremote.init();
// MIDIオブジェクトの初期化
	Midiout.begin();

// コールバック設定
	mVector.setFncHammer(&HammerSt,&HammerEd);
	mVector.setFncSwing(&Swinggg,&SwingEd);
	mVector.setModeHammer();

// モード初期化
	mMode = 0;
	pmWiiMidi = &mWMDSwingSample;
}
 
void loop()
{
  wiiremote.task(&myapp);
}
 
 
void myapp(void) {

	double x,y,z;

	x = (double) wiiremote.Report.Accel.X;
	y = (double) wiiremote.Report.Accel.Y;
	z = (double) wiiremote.Report.Accel.Z;

// 振り回し検出
	mVector.set(x,y,z);

// モード別タスク呼び出し
	pmWiiMidi->task();


// ±ボタンクリックによる、モードの切り替え
	{
		uint8_t	ledset;
		char 	tmpMode = mMode;
		if(wiiremote.buttonClicked(WIIREMOTE_PLUS))
		{	mMode = min(mMode+1,2);	}
		if(wiiremote.buttonClicked(WIIREMOTE_MINUS))
		{	mMode = max(mMode-1,0);	}
		if(tmpMode != mMode)
		{
			
			switch(mMode)
			{
			case 0:// mode A
				ledset = WIIREMOTE_LED1;
				pmWiiMidi = &mWMDSwingSample;
				mVector.setModeHammer();
				break;
			case 1:// mode B
				ledset = WIIREMOTE_LED2;
				pmWiiMidi = &mWMDDrumRoll;
//				mVector.setModeSwing();
				break;
			case 2:// mode C
				ledset = WIIREMOTE_LED3;
				pmWiiMidi = &mWMDEffect;
//				mVector.setModeHammer();
				break;
			default:
				ledset = WIIREMOTE_LED1 | WIIREMOTE_LED4;
				break;
			}
			wiiremote.setLED(ledset);
		}
	}

} // myapp

// 振り回し/振り下ろし対応
void HammerSt(void)
{
	pmWiiMidi->HammerSt();
}
void HammerEd(void)
{
	pmWiiMidi->HammerEd();
}
void Swinggg(int i)
{
	pmWiiMidi->Swinggg(i);
}
void SwingEd(void)
{
	pmWiiMidi->SwingEd();
}

